@@ -31,7 +31,7 @@ renv_available_packages_impl <- function(type, quiet = FALSE) {
   repos <- getOption("repos")
   urls <- contrib.url(repos, type)
   errors <- new.env(parent = emptyenv())
-  dbs <- lapply(urls, renv_available_packages_query, type = type, errors = errors)
+  dbs <- lapply(urls, renv_available_packages_query, errors = errors)
   names(dbs) <- names(repos)
 
   # notify finished
@@ -77,7 +77,7 @@ renv_available_packages_query_packages <- function(url) {
   suppressWarnings(read.dcf(destfile))
 }
 
-renv_available_packages_query <- function(url, type, errors) {
+renv_available_packages_query <- function(url, errors) {
 
   # check for a cached value
   name <- sprintf("repos_%s.rds.cache", URLencode(url, reserved = TRUE))
@@ -225,12 +225,54 @@ renv_available_packages_latest_impl <- function(package, type) {
   dbs <- renv_available_packages(type = type, quiet = TRUE)
   fields <- c("Package", "Version", "NeedsCompilation", "Repository")
   entries <- bapply(dbs, function(db) {
-    db[db$Package == package, intersect(fields, names(db))]
+
+    # extract entries for this package
+    rows <- db[db$Package == package, ]
+    if (nrow(rows) == 0L)
+      return(rows)
+
+    # only keep entries for which this version of R is compatible
+    deps <- rows$Depends %||% rep.int("", nrow(rows))
+    compatible <- map_lgl(deps, function(dep) {
+
+      # read 'R' entries from Depends (if any)
+      parsed <- catch(renv_description_parse_field(dep))
+      if (inherits(parsed, "error")) {
+        warning(parsed)
+        return(FALSE)
+      }
+
+      # read requirements for R
+      r <- parsed[parsed$Package == "R", ]
+      if (nrow(r) == 0)
+        return(TRUE)
+
+      # build code to validate requirements
+      fmt <- "getRversion() %s \"%s\""
+      all <- sprintf(fmt, r$Require, r$Version)
+      code <- paste(all, collapse = " && ")
+
+      # evaluate it
+      status <- catch(eval(parse(text = code), envir = baseenv()))
+      if (inherits(status, "error")) {
+        warning(status)
+        return(TRUE)
+      }
+
+      # all done
+      status
+
+    })
+
+    # keep only compatible rows + the required fields
+    rows[compatible, intersect(fields, names(db))]
+
   }, index = "Name")
 
   if (is.null(entries))
     return(NULL)
 
+  # sort based on version
   version <- numeric_version(entries$Version)
   ordered <- order(version, decreasing = TRUE)
   entries[ordered[[1]], ]
