@@ -56,7 +56,7 @@ hydrate <- function(packages = NULL,
                     project = NULL)
 {
   renv_scope_error_handler()
-  renv_dots_disallow(...)
+  renv_dots_check(...)
   project  <- renv_project_resolve(project)
   library  <- library %||% renv_libpaths_default()
 
@@ -124,19 +124,14 @@ renv_hydrate_dependencies <- function(project,
 # on CRAN but not that we want to use during tests
 renv_hydrate_libpaths <- function() {
 
-  config <- renv_config("hydrate.libpaths", default = character())
-  if (!is.character(config)) {
-    warning("ignoring non-character 'hydrate.libpaths' option")
-    config <- character()
-  }
-
-  if (is.character(config) && length(config))
-    config <- unlist(strsplit(config, ":", fixed = TRUE))
+  conf <- config$hydrate.libpaths()
+  if (is.character(conf) && length(conf))
+    conf <- unlist(strsplit(conf, ":", fixed = TRUE))
 
   libpaths <- if (renv_testing())
     renv_libpaths_all()
-  else if (length(config))
-    config
+  else if (length(conf))
+    conf
   else
     c(renv_libpaths_user(), renv_libpaths_site(), renv_libpaths_system())
 
@@ -152,7 +147,7 @@ renv_hydrate_link_package <- function(package, location, library) {
 
   # construct path to cache
   record <- renv_snapshot_description(location)
-  cache <- renv_cache_package_path(record)
+  cache <- renv_cache_find(record)
   if (!nzchar(cache))
     return(FALSE)
 
