@@ -4,6 +4,7 @@
 # NOTE: if sandboxing is used then these symbols will be clobbered;
 # save them so we can properly restore them later if so required
 renv_libpaths_init <- function() {
+  assign(".libPaths()",   .libPaths(),   envir = `_renv_libpaths`)
   assign(".Library",      .Library,      envir = `_renv_libpaths`)
   assign(".Library.site", .Library.site, envir = `_renv_libpaths`)
 }
@@ -25,15 +26,9 @@ renv_libpaths_site <- function() {
 }
 
 renv_libpaths_external <- function(project) {
-
   projlib <- settings$external.libraries(project = project)
-
-  conflib <- renv_config("external.libraries", default = character())
-  if (is.function(conflib))
-    conflib <- conflib(project = project)
-
+  conflib <- config$external.libraries(project)
   .expand_R_libs_env_var(c(projlib, conflib))
-
 }
 
 # on Windows, attempting to use a library path containing
@@ -140,7 +135,7 @@ renv_libpaths_activate <- function(project) {
 
   projlib <- renv_paths_library(project = project)
   extlib <- renv_libpaths_external(project = project)
-  userlib <- if (renv_config("user.library", default = FALSE))
+  userlib <- if (config$user.library())
     renv_libpaths_user()
 
   libpaths <- c(projlib, extlib, userlib)
@@ -150,16 +145,7 @@ renv_libpaths_activate <- function(project) {
 
 }
 
-renv_libpaths_save <- function() {
-  libpaths <- renv_global_get("default.libpaths")
-  if (is.null(libpaths))
-    renv_global_set("default.libpaths", libpaths)
-}
-
 renv_libpaths_restore <- function() {
-  libpaths <- renv_global_get("default.libpaths")
-  if (!is.null(libpaths)) {
-    renv_global_clear("default.libpaths")
-    renv_libpaths_set(libpaths)
-  }
+  libpaths <- get(".libPaths()", envir = `_renv_libpaths`)
+  renv_libpaths_set(libpaths)
 }
