@@ -3,15 +3,18 @@ renv_parse <- function(file = "", text = NULL, encoding = "UTF-8", ...) {
 
   renv_scope_options(warn = -1L)
 
+  # if text was provided, use it (ignoring file)
   if (is.character(text)) {
-    srcfile <- ifelse(nzchar(file), file, NULL)
+    srcfile <- if (nzchar(file)) file
     return(parse(text = text, encoding = encoding, srcfile = srcfile, ...))
   }
 
-  if (nzchar(file))
-    return(parse(file = file, encoding = encoding, ...))
+  # if file is an empty string, ignore it and return NULL
+  if (!nzchar(file))
+    return(NULL)
 
-  NULL
+  # otherwise, attempt to parse
+  return(parse(file = file, encoding = encoding, ...))
 
 }
 
